@@ -20,15 +20,16 @@
 #include <bonobo/bonobo-widget.h>
 
 #include <gtk/gtkmain.h>
-#include <gtk/gtkclist.h>
+#include <gtk/gtktreeview.h>
+#include <gtk/gtktreeselection.h>
 #include <gtk/gtkfilesel.h>
 #include <gtk/gtksignal.h>
 #include <gtk/gtkeditable.h>
 
 #include <bonobo/bonobo-i18n.h>
 
-#define GET_MODE(w) (GPOINTER_TO_INT (gtk_object_get_data (GTK_OBJECT (w), "GnomeFileSelectorMode")))
-#define SET_MODE(w, m) (gtk_object_set_data (GTK_OBJECT (w), "GnomeFileSelectorMode", GINT_TO_POINTER (m)))
+#define GET_MODE(w) (GPOINTER_TO_INT (g_object_get_data (G_OBJECT (w), "GnomeFileSelectorMode")))
+#define SET_MODE(w, m) (g_object_set_data (G_OBJECT (w), "GnomeFileSelectorMode", GINT_TO_POINTER (m)))
 
 typedef enum {
 	FILESEL_OPEN,
@@ -36,6 +37,13 @@ typedef enum {
 	FILESEL_SAVE
 } FileselMode;
 
+/* Take in sync with gtkfilesel.c */
+enum {
+  FILE_COLUMN
+};
+
+static GQuark user_data_id = 0;
+
 static gint
 delete_file_selector (GtkWidget *d, GdkEventAny *e, gpointer data)
 {
@@ -77,10 +85,12 @@ listener_cb (BonoboListener *listener,
 		for (i = 0; i < seq->_length; i++)
 			strv[i] = g_strdup (seq->_buffer[i]);
 		strv[i] = NULL;
-		gtk_object_set_user_data (GTK_OBJECT (dialog), strv);
+		g_object_set_qdata (G_OBJECT (dialog),
+				    user_data_id, strv);
 	} else 
-		gtk_object_set_user_data (GTK_OBJECT (dialog),
-					  g_strdup (seq->_buffer[0]));
+		g_object_set_qdata (G_OBJECT (dialog),
+				    user_data_id,
+				    g_strdup (seq->_buffer[0]));
 
  cancel_clicked:
 	g_free (subtype);
@@ -127,7 +137,7 @@ create_bonobo_selector (gboolean    enable_vfs,
 
 	dialog = gtk_window_new (GTK_WINDOW_TOPLEVEL);
 	gtk_container_add (GTK_CONTAINER (dialog), GTK_WIDGET (control));
-	gtk_widget_set_usize (dialog, 560, 450);
+	gtk_window_set_default_size (GTK_WINDOW (dialog), 560, 450);
 
 	bonobo_event_source_client_add_listener (
 		bonobo_widget_get_objref (control), 
@@ -191,35 +201,42 @@ ok_clicked_cb (GtkWidget *widget, gpointer data)
 		}
 		gtk_file_selection_set_filename (fsel, dir_name);
 		g_free (dir_name);
+
 	} else if (GET_MODE (fsel) == FILESEL_OPEN_MULTI) {
-		GtkCList *clist;
-		GList  *row;
-		char **strv;
+
+		GtkTreeSelection *selection;
+		GtkTreeModel *model;
+		GtkTreeIter iter;
+		
 		char *filedirname;
-		int i, rows, rownum;
+		char **strv;
+		int rows, i;
 
 		gtk_widget_hide (GTK_WIDGET (fsel));
-		
-		clist = GTK_CLIST (fsel->file_list);
-		rows = g_list_length (clist->selection);
-		
+
+		model = gtk_tree_view_get_model (GTK_TREE_VIEW (GTK_FILE_SELECTION (fsel)->file_list));	
+		selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (GTK_FILE_SELECTION (fsel)->file_list));	
+
+		rows = 	gtk_tree_model_iter_n_children (model, NULL);
+
 		strv = g_new (char *, rows + 2);
 		strv[rows] = g_strdup (file_name);
 
-		filedirname = g_dirname (file_name);
+		if (!gtk_tree_model_get_iter_root (model, &iter))
+			return;
 
-		rownum = 0;
-		i = 0;
-		row = clist->row_list;
+		filedirname = g_path_get_dirname (file_name);
 		
-		for ( ; row; row = g_list_next (row)) {
-			
-			if ((GTK_CLIST_ROW (row)->state == GTK_STATE_SELECTED) &&
-			    (gtk_clist_get_cell_type (clist, rownum, 0) == GTK_CELL_TEXT)) {		
-				gchar* f;
-
-				gtk_clist_get_text (clist, rownum, 0, &f);
+		i = 0;
+		
+		do {
+			if (gtk_tree_selection_iter_is_selected (selection, &iter)) {
+		      		gchar *f;
+      
+      				gtk_tree_model_get (model, &iter, FILE_COLUMN, &f, -1);
 				strv[i] = concat_dir_and_file (filedirname, f);
+				
+				g_free (f);
 
 				/* avoid duplicates */
 				if (strv[rows] && (strcmp (strv[i], strv[rows]) == 0)) {
@@ -229,23 +246,25 @@ ok_clicked_cb (GtkWidget *widget, gpointer data)
 
 				++i;
 			}
+		} while (gtk_tree_model_iter_next (model, &iter));
 
-			++rownum;
-		}
-		
-		/* g_assert (i == rows); */
-		
-		strv[rows + 1] = NULL;
+		strv[i] = strv[rows];
+		strv[i + 1] = NULL;
+
+		strv = g_renew (char *, strv, i + 2);
 
 		g_free (filedirname);
 
-		gtk_object_set_user_data (GTK_OBJECT (fsel), strv);
+		g_object_set_qdata (G_OBJECT (fsel),
+				    user_data_id, strv);
 		gtk_main_quit ();
+
 	} else {
 		gtk_widget_hide (GTK_WIDGET (fsel));
 
-		gtk_object_set_user_data (GTK_OBJECT (fsel),
-					  g_strdup (file_name));
+		g_object_set_qdata (G_OBJECT (fsel),
+				    user_data_id,
+				    g_strdup (file_name));
 		gtk_main_quit ();
 	}
 }
@@ -269,12 +288,12 @@ create_gtk_selector (FileselMode mode,
 
 	filesel = gtk_file_selection_new (NULL);
 
-	gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (filesel)->ok_button),
-			    "clicked", GTK_SIGNAL_FUNC (ok_clicked_cb),
+	g_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (filesel)->ok_button),
+			    "clicked", G_CALLBACK (ok_clicked_cb),
 			    filesel);
 
-	gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (filesel)->cancel_button),
-			    "clicked", GTK_SIGNAL_FUNC (cancel_clicked_cb),
+	g_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (filesel)->cancel_button),
+			    "clicked", G_CALLBACK (cancel_clicked_cb),
 			    filesel);
 
 	if (default_path)
@@ -300,8 +319,13 @@ create_gtk_selector (FileselMode mode,
 	g_free (path);
 
 	if (mode == FILESEL_OPEN_MULTI) {
-		gtk_clist_set_selection_mode (GTK_CLIST (GTK_FILE_SELECTION (filesel)->file_list),
-					      GTK_SELECTION_EXTENDED);
+		GtkTreeSelection *selection;
+
+		selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (GTK_FILE_SELECTION (filesel)->file_list));	
+		/* FIXME: change GTK_SELECTION_SINGLE to GTK_SELECTION_MULTIPLE as soon as bug #70505
+		 * will be fixed -- Paolo
+		 */	
+		gtk_tree_selection_set_mode (selection, GTK_SELECTION_SINGLE);
 	}
 
 	return GTK_WINDOW (filesel);
@@ -320,6 +344,9 @@ run_file_selector (GtkWindow  *parent,
 	GtkWindow *dialog = NULL;
 	gpointer   retval;
 
+	if (!user_data_id)
+		user_data_id = g_quark_from_static_string ("UserData");
+
 	if (!g_getenv ("GNOME_FILESEL_DISABLE_BONOBO"))
 		dialog = create_bonobo_selector (enable_vfs, mode, mime_types, 
 						 default_path, default_filename);
@@ -334,15 +361,15 @@ run_file_selector (GtkWindow  *parent,
 	if (parent)
 		gtk_window_set_transient_for (dialog, parent);
 	
-	gtk_signal_connect (GTK_OBJECT (dialog), "delete_event",
-			    GTK_SIGNAL_FUNC (delete_file_selector),
+	g_signal_connect (GTK_OBJECT (dialog), "delete_event",
+			    G_CALLBACK (delete_file_selector),
 			    dialog);
 
 	gtk_widget_show_all (GTK_WIDGET (dialog));
 
 	gtk_main ();
 
-	retval = gtk_object_get_user_data (GTK_OBJECT (dialog));
+	retval = g_object_get_qdata (G_OBJECT (dialog), user_data_id);
 
 	gtk_widget_destroy (GTK_WIDGET (dialog));
 
